@@ -9,7 +9,7 @@
     Released under the GNU Public Licence, Version 2.
 */
 
-#define VERSION "2.4.2"
+#define VERSION "2.4.3"
 
 #include "ksymoops.h"
 #include <ctype.h>
@@ -56,9 +56,8 @@ regmatch_t *re_unbracketed_address_pmatch;
 
 static void usage(void)
 {
-    printf("Version " VERSION "\n");
-    printf("usage: %s\n", prefix);
-    printf("\t[-v vmlinux] [--vmlinux=vmlinux]\n\t\t\t\t\tWhere to read vmlinux\n"
+    static const char usage_text[] = 
+	   "\t[-v vmlinux] [--vmlinux=vmlinux]\n\t\t\t\t\tWhere to read vmlinux\n"
 	   "\t[-V] [--no-vmlinux]\t\tNo vmlinux is available\n"
 	   "\t[-k ksyms] [--ksyms=ksyms]\tWhere to read ksyms\n"
 	   "\t[-K] [--no-ksyms]\t\tNo ksyms is available\n"
@@ -79,11 +78,12 @@ static void usage(void)
 	   "\t[-h] [--help]\t\t\tPrint help text\n"
 	   "\t[-t target] [--target=target]\tTarget of oops log\n"
 	   "\t[-a architecture] [--architecture=architecture]\n\t\t\t\t\tArchitecture of oops log\n"
+	   "\t[-A \"address list\"] [--addresses=\"address list\"]\n\t\t\t\t\tAdhoc addresses to decode\n"
 	   "\t< Oops.file\t\t\tOops report to decode\n"
 	   "\n"
 	   "\t\tAll flags can occur more than once.  With the exception "
-		"of -o\n"
-	   "\t\tand -d which are cumulative, the last occurrence of each "
+		"of -o,\n"
+	   "\t\t-d and -A which are cumulative, the last occurrence of each "
 		"flag is\n"
 	   "\t\tused.  Note that \"-v my.vmlinux -V\" will be taken as "
 		"\"No vmlinux\n"
@@ -145,7 +145,10 @@ static void usage(void)
 #endif
 	   "\n"
 	   "\n"
-	  );
+	  ;
+    printf("Version " VERSION "\n");
+    printf("usage: %s\n", prefix);
+    printf("%s", usage_text);
 }
 
 /* Check if possibly conflicting options were specified */
@@ -243,6 +246,46 @@ static void spec_or_default(int spec, int *some_spec) {
 	printf(" (default)\n");
 }
 
+/* Extract adhoc addresses from a string and save for later.
+ * An address is a string of at least 4 hex digits delimited by white space or
+ * punctuation marks, possibly prefixed by 0x or 0X.
+ */
+static void adhoc_addresses(struct options *options, const unsigned char *string)
+{
+    int count, start;
+    const unsigned char *p = string, *p1;
+    char *address;
+    while (*p) {
+	while (*p && !isxdigit(*p))
+	    ++p;
+	if (!*p)
+	    continue;
+	start = p == string || isspace(*(p-1)) || ispunct(*(p-1));
+	if (!start && p >= string+2 && *(p-2) == '0' && (*(p-1) == 'x' || *(p-1) == 'X')) {
+	    start = p == string+2 || isspace(*(p-3)) || ispunct(*(p-3));
+	}
+	if (!start) {
+	    ++p;
+	    continue;
+	}
+	p1 = p;
+	while (isxdigit(*++p)) {};
+	if (p-p1 < 4)
+	    continue;
+	address = malloc(p-p1+1);
+	if (!address)
+	    malloc_error("adhoc_addresses");
+	memcpy(address, p1, p-p1);
+	address[p-p1] = '\0';
+	for (count = 0; options->adhoc_addresses && options->adhoc_addresses[count]; ++count) {};
+	options->adhoc_addresses = realloc(options->adhoc_addresses, (count+2)*sizeof(options->adhoc_addresses));
+	if (!options->adhoc_addresses)
+	    malloc_error("adhoc_addresses");
+	options->adhoc_addresses[count] = address;
+	options->adhoc_addresses[count+1] = NULL;
+    }
+}
+
 /* Parse the options.  Verbose but what's new with getopt? */
 static void parse(int argc, char **argv, struct options *options, int *spec_h)
 {
@@ -277,13 +320,14 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
 	{"help", 0, 0, 'h'},
 	{"target", 1, 0, 't'},
 	{"architecture", 1, 0, 'a'},
+	{"addresses", 1, 0, 'A'},
 	{0, 0, 0, 0}
     };
 
     int c, i, some_spec = 0;
     char *p, *before;
 
-    while ((c = getopt_long(argc, argv, "v:Vk:Kl:Lo:Om:Ms:e1xSiIT:hdt:a:",
+    while ((c = getopt_long(argc, argv, "v:Vk:Kl:Lo:Om:Ms:e1xSiIT:hdt:a:A:",
 	    long_opts, NULL)) != EOF) {
 	if (c != 'd')
 	    DEBUG(1, "'%c' '%s'", c, optarg);
@@ -393,6 +437,9 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
 	case 'a':
 	    options->architecture = optarg;
 	    break;
+	case 'A':
+	    adhoc_addresses(options, optarg);
+	    break;
 	case '?':
 	    if (c == 'c')
 		printf("Option -c is obsolete, use -e toggle instead\n");
@@ -404,6 +451,13 @@ static void parse(int argc, char **argv, struct options *options, int *spec_h)
     options->filecount = argc - optind;
     options->filename = argv + optind;
 
+    if (options->adhoc_addresses && !options->filecount) {
+	static char *null = "/dev/null";
+	static char **null_list = { &null };
+	options->filecount = 1;
+	options->filename = null_list;
+    }
+
     /* Expand any requests for the current uname values */
     convert_uname(&options->vmlinux);
     if (options->objects) {
@@ -617,6 +671,7 @@ int main(int argc, char **argv)
 	0,	/* truncate */
 	NULL,   /* target */
 	NULL,   /* architecture */
+	NULL,   /* addresses */
 	0	/* address_bits */
     };
     static char const procname[] = "main";
