@@ -1639,6 +1639,7 @@ static int Oops_print(const char *line, const char **text, char ***string,
     /* various */   "|In swapper task"
     /* various */   "|kmem_free"
     /* various */   "|swapper"
+    /* various */   "|Pid:"
 
     /* i386 2.0 */  "|Corrupted stack page"
     /* i386 */	    "|invalid operand: "
@@ -2063,7 +2064,8 @@ static FILE *Oops_next_file(OPTIONS *options)
 	if (f)
 	    fclose_local(f, procname);
 	f = NULL;
-	if (regular_file(*(options->filename), procname))
+	if (strcmp(*(options->filename), "/dev/null") == 0 || 
+	    regular_file(*(options->filename), procname))
 	    f = fopen_local(*(options->filename), "r", procname);
 	if (f)
 	    DEBUG(1, "reading Oops report from %s", *(options->filename));
@@ -2092,6 +2094,13 @@ int Oops_read(OPTIONS *options)
     ss_init(&ss_format, "Oops log data");
     Oops_open_input_bfd(&me, &ibfd, options);
 
+    if (options->adhoc_addresses) {
+	char **adhoc;
+	for (adhoc = options->adhoc_addresses; *adhoc; ++adhoc) {
+	    add_symbol(&ss_format, *adhoc, 'A', 1, "Adhoc");
+	}
+    }
+
     if (!options->filecount && isatty(0))
 	printf("Reading Oops report from the terminal\n");
 
@@ -2193,7 +2202,7 @@ int Oops_read(OPTIONS *options)
 		strstr(options->target, "sparc") == 0 &&
 		sparc_regdump)
 		;	/* sparc regdump comes *after* code, ignore :( */
-	    else
+	    else if (!options->adhoc_addresses)
 		WARNING("%s", "Code line not seen, dumping what data is available");
 	    Oops_format(&ss_format, options);
 	    ss_free(&ss_format);
